Package: robmixglm
Type: Package
Title: Robust Generalized Linear Models (GLM) using Mixtures
Version: 1.2-5
Date: 2025-10-31
Authors@R: c(person("Ken","Beath",email="ken@kjbeath.id.au", role=c("aut","cre")))
Maintainer: Ken Beath <ken@kjbeath.id.au>
Contact: Ken Beath <ken@kjbeath.id.au>
Author: Ken Beath [aut, cre]
Description: Robust generalized linear models (GLM) using a mixture method, as described in Beath (2018) <doi:10.1080/02664763.2017.1414164>. This assumes that the data are a mixture of standard observations, being a generalised linear model, and outlier observations from an overdispersed generalized linear model. The overdispersed linear model is obtained by including a normally distributed random effect in the linear predictor of the generalized linear model.
Depends: R(>= 3.2.0)
Suggests: R.rsp, robustbase, lattice, forward
VignetteBuilder: R.rsp
Imports: fastGHQuad, stats, bbmle, VGAM, actuar, Rcpp (>= 0.12.15),
        methods, boot, numDeriv, parallel, doParallel, foreach, doRNG,
        MASS
LazyLoad: yes
LazyData: yes
NeedsCompilation: yes
License: GPL (>= 2)
LinkingTo: Rcpp
Packaged: 2025-10-31 06:21:35 UTC; kjbea
Repository: CRAN
Date/Publication: 2025-10-31 09:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 19:53:55 UTC; unix
Archs: robmixglm.so.dSYM

Package: robmixglm
Type: Package
Title: Robust Generalized Linear Models (GLM) using Mixtures
Version: 1.2-8
Date: 2026-07-02
Authors@R: c(person("Ken","Beath",email="ken@kjbeath.id.au", role=c("aut","cre")))
Maintainer: Ken Beath <ken@kjbeath.id.au>
Contact: Ken Beath <ken@kjbeath.id.au>
Author: Ken Beath [aut, cre]
Description: Robust generalized linear models (GLM) using a mixture method, as described in Beath (2018) <doi:10.1080/02664763.2017.1414164>. This assumes that the data are a mixture of standard observations, being a generalised linear model, and outlier observations from an overdispersed generalized linear model. The overdispersed linear model is obtained by including a normally distributed random effect in the linear predictor of the generalized linear model.
Depends: R(>= 3.2.0)
Suggests: R.rsp, robustbase, lattice, forward
VignetteBuilder: R.rsp
Imports: fastGHQuad, stats, bbmle, VGAM, actuar, Rcpp (>= 0.12.15),
        methods, boot, numDeriv, parallel, MASS
LazyLoad: yes
LazyData: yes
NeedsCompilation: yes
License: GPL (>= 2)
LinkingTo: Rcpp
Packaged: 2026-07-02 09:43:45 UTC; kenbeath
Repository: CRAN
Date/Publication: 2026-07-02 10:00:03 UTC

useDynLib(robmixglm, .registration=TRUE)
# exportPattern("^[[:alpha:]]+")
importFrom(Rcpp, evalCpp)

export(robmixglm)
export(coef.robmixglm)
export(summary.robmixglm)
export(AIC.robmixglm)
export(BIC.robmixglm)
export(logLik.robmixglm)
export(outlierProbs)
export(outlierProbs.robmixglm)
export(outlierTest)
export(outlierTest.robmixglm)
export(print.outlierTest)
export(summary.outlierTest)
export(print.summary.outlierTest)
export(extractAIC.robmixglm)
export(fitted.robmixglm)
export(predict.robmixglm)

# Imported functions
importFrom("stats", "dbeta", "dnorm", "nlm", "pnorm", "qnorm", "rnorm", "runif","as.formula","fitted","terms","predict")
importFrom("stats", "AIC", "BIC", "logLik","optim")
importFrom("stats", "model.extract", "model.matrix", "model.response","weighted.mean")
importFrom("stats", "dpois", "nlminb", "optimHess", "rbinom", "rgamma", "dbinom", "dgamma", "dnbinom")
importFrom("stats", "Gamma", "binomial", "constrOptim", "gaussian",
             "glm.fit", "poisson", "printCoefmat", "rpois","rnbinom","delete.response")
importFrom("stats", ".getXlevels", "model.frame")
importFrom("stats", "glm", "lm")

importFrom("MASS", "ginv")
importFrom("methods", "signature")
 
importFrom("fastGHQuad", "gaussHermiteData")

importFrom("bbmle", "mle2", "parnames", "parnames<-", "coef")

importFrom("VGAM", "vglm", "pospoisson")

importFrom("actuar", "dztpois")

importFrom("graphics", "abline", "dotchart")

importFrom("boot", "boot")

importFrom("numDeriv", "hessian")

importFrom("parallel", "detectCores")
# ,"makeCluster","stopCluster"
#importFrom("doParallel", "registerDoParallel")

#importFrom("foreach", "foreach", "%dopar%")

# S3 methods
S3method(logLik, robmixglm)
S3method(AIC, robmixglm)
S3method(BIC, robmixglm)
S3method(coef, robmixglm)
S3method(summary, robmixglm)
S3method(outlierProbs, robmixglm)
S3method(extractAIC, robmixglm)
S3method(fitted, robmixglm)
S3method(residuals, robmixglm)
S3method(predict, robmixglm)

S3method(plot,outlierProbs)

S3method(print, summary.robmixglm)

S3method(outlierTest, robmixglm)
S3method(print, outlierTest)
S3method(summary, outlierTest)
S3method(print,summary.outlierTest)

print.summary.robmixglm <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
  if (!inherits(x, "summary.robmixglm"))
    stop("Use only with 'summary.robmixglm' objects.\n")
  printCoefmat(x$coefficients,digits=digits,na.print="")
  cat("\n")
  print(data.frame(logLik = x$logLik,AIC = x$AIC, BIC = x$BIC,
                   row.names = " "))
  invisible(x)
 }

myoptim <- function(par, fn, gr,method,
                          lower, upper,
                          control, hessian,...)
{
  myfn <- function(par) {
    temp <- fn(par)
    if (!is.finite(temp)) temp <- 1.0e100
    return(temp)
  }
  if (is.null(control$trace)) verbose <- FALSE
  else verbose <- control$trace

    if (verbose) cat("using nlminb","\n")
  if (length(lower) < length(par)) lower <- rep(lower,length(par))
  if (length(upper) < length(par)) upper <- rep(upper,length(par))
  nlmcontrol <- control
  thenlm <- suppressWarnings(nlminb(par, fn,lower = lower, upper = upper, control=nlmcontrol))
   if (thenlm$convergence==0) theoptim <- list(par=thenlm$par,value=thenlm$objective,convergence=thenlm$convergence,message=thenlm$message)
  else {
    if (verbose) cat("using optim","\n")
    optimcontrol <- list(trace=ifelse(verbose,1,0),maxit=control$iter.max)
    theoptim <- suppressWarnings(optim(thenlm$par, myfn, method="L-BFGS-B",lower=lower,upper=upper,hessian=FALSE,control=optimcontrol))
  }
  if (theoptim$convergence!=0)  {
    if (verbose) cat("using constrOptim","\n")
    ui <- diag(length(par))[is.finite(lower),]
    ci <- lower[is.finite(lower)]
    theoptim <- suppressWarnings(constrOptim(par, fn, NULL , ui, ci, control=optimcontrol))
  }
  return(theoptim)
}

fitted.robmixglm <- function(object, ...) {
  if (!inherits(object, "robmixglm"))
    stop("Use only with 'robmixglm' objects.\n")
    lp <- object$X %*% matrix(coef(object),ncol=1) + object$offset
  out <- switch (
    object$family,
    gaussian = lp,
    binomial = 1.0/(1.0+exp(-lp)),
    poisson = exp(lp),
    gamma = exp(lp),
    truncpoisson = exp(lp),
    nbinom = exp(lp)
  )
  return(out)
}

negll <- function(p, y, x, offset) {
  lp <- x %*% matrix(p[1:(length(p)-1)],ncol=1)+offset
  theta <- p[length(p)]
  negll <- -sum(dnbinom(y,mu=exp(lp),size=theta,log=TRUE))
  if (negll>1.0e100) negll <- 1.0e100
  if (!is.finite(negll)) stop("Error in calculation log likelihood")
  return(negll)
} 

fitnegbin <- function(y,x,offset) {
  # obtain starting values
  pois.glm <- glm(y~-1+x, family=poisson, offset=offset)
  initp <- c(coef(pois.glm),1)
  fit <- nlminb(initp, negll, lower = c(rep(-Inf,length(initp)-1),0),   x=x, y=y, offset=offset)
# allow for singular convergence
  if (fit$convergence>1) stop("Convergence failed")
  return(fit)
}

poisson.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TRUE, cores, verbose,   starting.values=NULL) {
  
  if (!is.null(starting.values)) {
    notrials <- 1
    cores <- 1
  }
  
  
  llrandpois <- function(y, lp, tau2, gh){
    
    #browser()
    
    calconell <- function(x) {
      oney <- x[1]
      onelp <- x[2]
      tau2 <- x[3]
      if (tau2<0.0) return(NA)
      if (tau2==0) thell <- dpois(oney,exp(onelp),TRUE) else
        thell <- llrandpoiscpp(y, lp, tau2, gh)
      return(thell)
    }
    if (is.na(tau2)) stop("tau2 is NA")
    ll <- apply(cbind(y,lp),1,calconell)
    return(ll)
  }
  
  thestatistic <- function(data, ...) {

    rlreg <- function(xcoef,lpoutlier,tau2,x,y,offset,prop) {
      poutlier <- 1.0/(1+exp(-lpoutlier))
      
      lp <- as.vector(x %*% xcoef)+offset
      
      lp1 <- dpois(y, exp(lp), TRUE)
      lp2 <- llrandpoiscpp(y, lp, tau2, gh)
      if (!missing(prop)) {
        ll <- prop*cbind(lp1,lp2)
        negll <- -sum(apply(ll,1,sum))
      } else {
        l <- exp(cbind(lp1+log(1-poutlier),lp2+log(poutlier)))
        negll <- -sum(log(apply(l,1,sum)))
      }
      if (is.nan(negll)) negll <- 1e108
      if (!is.finite(negll)) negll <- 1e108
      return(negll)
    }
    
    optimrlreg <- function(p,lpoutlier,x,y,offset,prop) {
      return(rlreg(matrix(p[1:(length(p)-1)],ncol=1),lpoutlier,p[length(p)],x,y,offset,prop))
      
    }
    
 
    #browser()
    
    y <- data[,1]
    x <- data[,3:dim(data)[2]]
    if (dim(data)[2] >=3) offset <- data[,2]
    else offset <- NULL
    
   tryCatch({
   if (is.null(starting.values)) {
     noutliers <- max(1,round(dim(data)[1]*0.2))
     outliers <- sample(c(rep(1,noutliers),rep(0,dim(data)[1]-noutliers)))
     
    robust.poisson.prefit <- glm(y~-1+x,
                                       family=poisson, offset=offset,subset=(outliers!=1))
    
    prefit.coef <- coef(robust.poisson.prefit)
    # assume 20% outliers as a starting point
    currlpoutlier <- log(0.2/(1-0.2))
    currxcoef <- matrix(prefit.coef[1:(length(prefit.coef))],ncol=1)
    currxcoef <- ifelse(is.na(currxcoef),0,currxcoef)
    currtau2 <- min(rgamma(1,1.5,1),2)
    } else {
      currxcoef <- matrix(starting.values[1:(length(starting.values)-2)],ncol=1)
      currlpoutlier <- starting.values[length(starting.values)-1]
      currtau2 <- starting.values[length(starting.values)]
    }
    currll <- -1.0e100
    nem <- 0
    
    repeat {
      nem <- nem+1
      # expectation step
      currpoutlier <- 1.0/(1+exp(-currlpoutlier))
      
      lp <- as.vector(x %*% currxcoef)+offset
 
      ll1 <- dpois(y, exp(lp), TRUE)+log(1-currpoutlier) 
      ll2 <- llrandpoiscpp(y, lp, currtau2, gh)+log(currpoutlier)
      
      ll <- cbind(ll1,ll2)
      prop <- t(apply(ll,1,function(x) {
        x <- x-max(x)
        x <- ifelse(x==-Inf,-1e100,x)
        return(exp(x)/sum(exp(x)))
      }))
      # calculate outlier proportion
      poutlier <- sum(prop[,2])/dim(prop)[1]
      currlpoutlier <- log(poutlier/(1-poutlier))
      
      if (is.na(poutlier)) stop("problem with poutlier")
      
      startvals <- c(currxcoef,currtau2)
      names(startvals) <- c(dimnames(x)[[2]],"tau2")
      
      names(startvals) <- c(dimnames(x)[[2]],"tau2")
      
      #browser() 
      
      results.nlm <- suppressWarnings(nlminb(startvals,optimrlreg,
                                             lower=c(rep(-Inf,length(currxcoef)),0),
                                             #control=list(trace=1,iter.max=5),
                                             control=list(iter.max=5),
                                             lpoutlier=currlpoutlier,
                                             prop=prop,
                                             y=y,x=x,offset=offset))
      
      currxcoef <- matrix(as.numeric(results.nlm$par)[1:(length(results.nlm$par)-1)],ncol=1)
      currtau2 <- as.numeric(results.nlm$par)[length(results.nlm$par)]
      
      lastll <- currll
      currll <- -rlreg(currxcoef,currlpoutlier,currtau2,x,y,offset)
      if (verbose) print(sprintf("Likelihood at end of EM step %.4f", currll))
      if (abs((lastll-currll)/currll)<EMTol) break()
      if (nem >100) break()
    }
    return(c(currll,currxcoef,currlpoutlier,currtau2))   
   },
   error=function(e) return(rep(NA,5))
   )
  }
  
  ll.robustpoisson <- function(p){
    
    xcoef <- p[1:(length(p)-2)]
    lpoutlier <- p[length(p)-1]
    tau2 <- p[length(p)]
    
    poutlier <- exp(lpoutlier)/(1+exp(lpoutlier))
    
    lp <- as.vector(x %*% xcoef)+offset
    
    ll1 <- dpois(y, exp(lp), TRUE)+log(1-poutlier)
    
    ll2 <- llrandpoiscpp(y, lp, tau2, gh)+log(poutlier)
    
    l <- exp(cbind(ll1,ll2))
    negll <- -sum(log(apply(l,1,sum)))
    if (is.nan(negll)) negll <- NA
    if (!is.finite(negll)) negll <- NA
    return(negll)
  }
  
  
 
  if (is.null(starting.values)) {
    bootdata <- cbind(y, offset, x)
  
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
  seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  
  
  if (cores>1) {
    if(.Platform$OS.type=="unix")  parallel <- "multicore"
    else parallel <- "snow" }
  else parallel <- "no"
  #browser()
  if (notrials == 1) {
    theboot <- matrix(thestatistic(bootdata), nrow=1)
  } else 
    theboot <- boot(bootdata, thestatistic, R=notrials, sim = "parametric",
                    ran.gen = function(d, p) d,
                    parallel = parallel,
                    ncpus = cores)$t
  
  res <- vector("list", length = notrials)
  for (i in 1:notrials) {
    res[[i]] <- list(logLik=theboot[i,1],start.val=theboot[i,2:dim(theboot)[2]])
  }
  
  maxll <- -Inf
  nfails <- 0
  
  #browser()
  for (i in 1:notrials) {
    if (verbose) cat(c(res[[i]]$logLik,res[[i]]$start.val),"\n")
    if (is.na(res[[i]]$logLik)) nfails <- nfails+1
    else {
      if (res[[i]]$logLik>maxll) {
        maxll <- res[[i]]$logLik
        start.val <- res[[i]]$start.val
      }
    }
    if (nfails > 0) warning(sprintf("Failed to obtain starting values for %i starting sets", nfails))
  }
  } else {
    start.val <- starting.values
  }
  
  #browser()
  
  if(is.null(start.val)) stop("Cannot find valid starting values") 
  
  thenames <- c(dimnames(x)[[2]],"lpoutlier","tau2")
  
  names(start.val) <- thenames
  
  parnames(ll.robustpoisson) <- names(start.val)
  
  lower.val <- c(rep(-Inf,length(start.val)-2),-Inf, 0)
  
  #browser()
  
  names(lower.val) <- names(start.val)
  
  
  if (verbose) thecontrol <- list(eval.max=1000,iter.max=1000,eval.max=2000,trace=5)
  else thecontrol <- list(eval.max=1000,iter.max=1000,eval.max=2000)
  
  robustpoisson.fit <- mle2(ll.robustpoisson,start=start.val,vecpar=TRUE,
                                 optimizer="user",optimfun=myoptim,
                                 data=list(y=y,x=x,offset=offset),
                                 skip.hessian=TRUE,trace=verbose,
                                 lower=lower.val,
                                 control=thecontrol)
  
  if (calcHessian) {
    thecoef <- coef(robustpoisson.fit)
    ncoef <- length(thecoef)
    if (thecoef[ncoef]<1e-4) thecoef[ncoef] <- 1e-4
    robustpoisson.fit@details$hessian <- optimHess(thecoef,ll.robustpoisson,control=list(ndeps=c(rep(1.0e-5,length(thecoef)))))
    robustpoisson.fit@vcov <- ginv(robustpoisson.fit@details$hessian)
  }
  if (any(is.nan(sqrt(diag(robustpoisson.fit@vcov))))) browser()
  
  xcoef <- matrix(coef(robustpoisson.fit)[1:(length(coef(robustpoisson.fit))-2)],ncol=1)
  lpoutlier <- coef(robustpoisson.fit)[length(coef(robustpoisson.fit))-1]
  poutlier <- 1.0/(1+exp(-lpoutlier))
  tau2 <- coef(robustpoisson.fit)[length(coef(robustpoisson.fit))]
  
  
  lp <- as.vector(x %*% xcoef)+offset
  
  ll1 <- dpois(y, exp(lp), TRUE)+log(1-poutlier) 
  ll2 <- llrandtruncpoiscpp(y, lp, tau2, gh)+log(poutlier)
  
  ll <- cbind(ll1,ll2)
  prop <- t(apply(ll,1,function(x) {
    x <- x-max(x)
    x <- ifelse(x==-Inf,-1e100,x)
    return(exp(x)/sum(exp(x)))
  }))
  
  coef.names <- c(dimnames(x)[[2]],"Outlier p.","Tau-sq")
  return(list(fit=robustpoisson.fit,prop=prop,logLik=-robustpoisson.fit@min,np=length(coef.names),nobs=dim(x)[1],coef.names=coef.names))  
} 

gaussian.outlierTest.robmixglm <- function(object, R, parallel, cores) {
  
  gaussian.rg <- function(data, mle) {
    
    # calculate linear predictor
    lp <- data$X %*% matrix(mle$coefficients,ncol=1) + data$offset
    sd <- sqrt(mle$deviance/mle$df.residual)
    data$Y <- rnorm(length(data$Y),mean=lp,sd=sd)
    return(data)
  }
  
  gaussian.fun <- function(thedata) {
    gaussian.mle2 <- glm.fit(thedata$X, thedata$Y, 
                             offset = thedata$offset, family = gaussian())
    thesd <- sqrt(gaussian.mle2$deviance/gaussian.mle2$df.residual)
    # starting values assume 20% outliers
    starting.values <- c(coef(gaussian.mle2)[1:length(coef(gaussian.mle2))],log(0.2/(1-0.2)),2*thesd,0.5*thesd)
    mixfit <- suppressWarnings(gaussian.fit.robmixglm(thedata$X, thedata$Y, 
                                       offset = thedata$offset,gh = norm.gauss.hermite(object$quadpoints),
                                       notrials=object$notrials, EMTol = object$EMTol, calcHessian=FALSE,
                                       verbose=FALSE, starting.values=starting.values, cores=1))
    lp <- thedata$X %*% matrix(gaussian.mle2$coefficients,ncol=1) + thedata$offset
    gaussian.loglik <- sum(dnorm(thedata$Y,mean=lp,sd=thesd,log=TRUE))
    sim.chisq <- 2*(mixfit$logLik-gaussian.loglik)
    fitno <<- fitno+1
    return(sim.chisq)
  }
  
  # fit glm to data
  gaussian.mle <- glm.fit(object$X, object$Y, 
                          offset = object$offset, family = gaussian())
  
  thedata <- list(X=object$X,Y=object$Y,offset=object$offset,mle=gaussian.mle)
  fitno <- 1
  gaussian.boot <- boot(thedata, gaussian.fun, R, sim="parametric",
                        ran.gen = gaussian.rg, mle=gaussian.mle,parallel=parallel,
                        ncpus = cores)
  return(gaussian.boot)
}

nbinom.outlierTest.robmixglm <- function(object, R, parallel, cores) {
  
  nbinom.rg <- function(data, mle) {
    
    # calculate linear predictor
    lp <- matrix(data$X,ncol=length(mle$par)-1) %*% matrix(mle$par[1:(length(mle$par)-1)],ncol=1) + data$offset
    theta <- mle$par[length(mle$par)]
    data$Y <- rnbinom(length(data$Y),mu=exp(lp),size=theta)
    return(data)
  }
  
  nbinom.fun <- function(data) {

    nbinom.mle2 <- fitnegbin(data$Y,object$X,data$offset)

    # starting values assume 20% outliers and tau^2 of 1
    starting.values <- c(nbinom.mle2$par[1:(length(nbinom.mle2$par)-1)],log(0.2/(1-0.2)),0.2,exp(nbinom.mle2$par[length(nbinom.mle2$par)]))
    mixfit <- suppressWarnings(nbinom.fit.robmixglm(data$X, data$Y, 
                                       offset = data$offset,gh = norm.gauss.hermite(object$quadpoints),
                                       notrials=object$notrials, EMTol = object$EMTol, calcHessian=FALSE,
                                       verbose=object$verbose, starting.values=starting.values, cores=1))
    

        lp <- data$X %*% matrix(nbinom.mle2$par[1:(length(nbinom.mle2$par)-1)],ncol=1) + data$offset
    
    nbinom.loglik <- sum(dnbinom(data$Y,mu=exp(lp),size=nbinom.mle2$par[length(nbinom.mle2$par)],log=TRUE))
    sim.chisq <- 2*(mixfit$logLik-nbinom.loglik)
    fitno <<- fitno+1
    return(sim.chisq)
  }

  nbinom.mle <- fitnegbin(object$Y,object$X,object$offset)
 
  thedata <- list(X=object$X,Y=object$Y,offset=object$offset,mle=nbinom.mle)
  fitno <- 1
  nbinom.boot <- boot(thedata, nbinom.fun, R, sim="parametric",
                        ran.gen = nbinom.rg, mle=nbinom.mle,parallel=parallel,
                      ncpus = cores)
  return(nbinom.boot)
}


gamma.outlierTest.robmixglm <- function(object, R, parallel, cores) {
  
  gamma.rg <- function(data, mle) {
    
    # calculate linear predictor
    lp <- data$X %*% matrix(mle$coefficients,ncol=1) + data$offset
    phi <- sum(mle$residuals^2)/mle$df.residual
     data$Y <- rgamma(length(data$Y),shape=1.0/phi,rate=1.0/(phi*exp(lp)))
    return(data)
  }
  
  gamma.fun <- function(thedata) {
    gamma.mle2 <- glm.fit(thedata$X, thedata$Y, 
                          offset = thedata$offset, family = Gamma(link="log"))
    # starting values assume 20% outliers and tau^2 of 1
    starting.values <- c(coef(gamma.mle2)[1:length(coef(gamma.mle2))],log(0.2/(1-0.2)),0.1,sum(gamma.mle2$residuals^2)/gamma.mle2$df.residual)
    mixfit <- suppressWarnings(gamma.fit.robmixglm(thedata$X, thedata$Y, 
                                    offset = thedata$offset,gh = norm.gauss.hermite(object$quadpoints),
                                    notrials=object$notrials, EMTol = object$EMTol, calcHessian=FALSE,
                                    verbose=FALSE, starting.values=starting.values, cores=1))
    lp <- thedata$X %*% matrix(gamma.mle2$coefficients,ncol=1) + thedata$offset
    phi <- sum(gamma.mle2$residuals^2)/gamma.mle2$df.residual
    gamma.loglik <- sum(dgamma(thedata$Y,shape=1.0/phi,rate=1.0/(phi*exp(lp)),log=TRUE))
    sim.chisq <- 2*(mixfit$logLik-gamma.loglik)
     fitno <<- fitno+1
    return(sim.chisq)
  }
  
  # fit glm to data
  gamma.mle <- glm.fit(object$X, object$Y, 
                       offset = object$offset, family = Gamma(link="log"))
  
  
  thedata <- list(X=object$X,Y=object$Y,offset=object$offset,mle=gamma.mle)
  fitno <- 1
  gamma.boot <- boot(thedata, gamma.fun, R, sim="parametric",
                        ran.gen = gamma.rg, mle=gamma.mle, parallel=parallel,
                     ncpus = cores)
  return(gamma.boot)
}

truncpoisson.outlierTest.robmixglm <- function(object, R, parallel, cores) {
  
  truncpoisson.rg <- function(data, mle) {
    
    lp <- data$X %*% matrix(mle@coefficients,ncol=1) + data$offset
    data$Y <- actuar::rztpois(length(data$Y),exp(lp))
    return(data)
  }
  
  truncpoisson.fun <- function(data) {
      
    truncpoisson.mle2 <- vglm(data$Y~data$X[,colnames(data$X)!="(Intercept)"],
                                       family=pospoisson, offset=data$offset)
    
    # starting values assume 20% outliers and tau^2 of 1
    starting.values <- c(coef(truncpoisson.mle2)[1:length(coef(truncpoisson.mle2))],log(0.2/(1-0.2)),0.1)
    mixfit <- suppressWarnings(truncpoisson.fit.robmixglm(data$X, data$Y, 
                                           offset = thedata$offset,gh = norm.gauss.hermite(object$quadpoints),
                                           notrials=object$notrials, EMTol = object$EMTol, calcHessian=FALSE,
                                           verbose=object$verbose, starting.values=starting.values, cores=1))
    lp <- data$X %*% matrix(truncpoisson.mle2@coefficients,ncol=1) + data$offset
    truncpoisson.loglik <- sum(dztpois(data$Y,exp(lp),log=TRUE))
    sim.chisq <- 2*(mixfit$logLik-truncpoisson.loglik)
    fitno <<- fitno+1
    return(sim.chisq)
  }
  
  
  truncpoisson.mle2 <- vglm(object$Y~object$X[,colnames(object$X)!="(Intercept)"],family=pospoisson, offset=object$offset)
  
  fitno <- 1
  
  thedata <- list(X=object$X,Y=object$Y,offset=object$offset,mle=truncpoisson.mle2)
  
  truncpoisson.boot <- boot(thedata, truncpoisson.fun, R, sim="parametric",
                            ran.gen = truncpoisson.rg, mle=truncpoisson.mle2, parallel=parallel,
                            ncpus = cores)
  return(truncpoisson.boot)
}



poisson.outlierTest.robmixglm <- function(object, R, parallel, cores) {

    poisson.rg <- function(data, mle) {
      
      # calculate linear predictor
      lp <- data$X %*% matrix(mle$coefficients,ncol=1) + data$offset
      data$Y <- rpois(length(data$Y),exp(lp))
      return(data)
    }

    poisson.fun <- function(thedata) {
      poisson.mle2 <- glm.fit(thedata$X, thedata$Y, 
                              offset = thedata$offset, family = poisson())
      # starting values assume 20% outliers and tau^2 of 1
      starting.values <- c(coef(poisson.mle2)[1:length(coef(poisson.mle2))],log(0.2/(1-0.2)),0.1)
      mixfit <- suppressWarnings(poisson.fit.robmixglm(thedata$X, thedata$Y, 
                                        offset = thedata$offset,gh = norm.gauss.hermite(object$quadpoints),
                                        notrials=object$notrials, EMTol = object$EMTol, calcHessian=FALSE,
                                        verbose=object$verbose, starting.values=starting.values, cores=1))
      lp <- thedata$X %*% matrix(poisson.mle2$coefficients,ncol=1) + thedata$offset
      poisson.loglik <- sum(dpois(thedata$Y,exp(lp),log=TRUE))
      sim.chisq <- 2*(mixfit$logLik-poisson.loglik)
    	fitno <<- fitno+1
      return(sim.chisq)
    }
    
    # fit glm to data
  poisson.mle <- glm.fit(object$X, object$Y, 
                         offset = object$offset, family = poisson())
  
 
  thedata <- list(X=object$X,Y=object$Y,offset=object$offset,mle=poisson.mle)
  fitno <- 1
  poisson.boot <- boot(thedata, poisson.fun, R, sim="parametric",
                       ran.gen = poisson.rg, mle=poisson.mle, parallel=parallel,
                       ncpus = cores)
  return(poisson.boot)
}

binomial.outlierTest.robmixglm <- function(object, R, parallel, cores) {
  
  binomial.rg <- function(data, mle) {
    # calculate linear predictor
    lp <- data$X %*% matrix(mle$coefficients,ncol=1) + data$offset
    n <- data$Y[,1]+data$Y[,2]
    data$Y[,1] <- rbinom(dim(data$Y)[1],data$Y[,1]+data$Y[,2],1.0/(1.0+exp(-lp)))
    data$Y[,2] <- n-data$Y[,1]
    return(data)
  }
  
  binomial.fun <- function(thedata) {
    binomial.mle2 <- glm.fit(thedata$X, thedata$Y,
                             offset = thedata$offset, family = binomial())
    starting.values <- c(coef(binomial.mle2),log(0.2/(1-0.2)),1)
    if (fitno==1) mixfit <- suppressWarnings(binomial.fit.robmixglm(thedata$X, thedata$Y,
                                       offset = thedata$offset,gh = norm.gauss.hermite(object$quadpoints),
                                       notrials=object$notrials, EMTol = object$EMTol, calcHessian=FALSE,
                                       verbose=FALSE, starting.values=NULL, cores=1))
    else mixfit <- suppressWarnings(binomial.fit.robmixglm(thedata$X, thedata$Y,
                                       offset = thedata$offset,gh = norm.gauss.hermite(object$quadpoints),
                                       notrials=object$notrials, EMTol = object$EMTol, calcHessian=FALSE,
                                       verbose=FALSE, starting.values=starting.values, cores=1))
    lp <- thedata$X %*% matrix(binomial.mle2$coefficients,ncol=1) + thedata$offset
    binomial.loglik <- sum(dbinom(thedata$Y[,1],thedata$Y[,1]+thedata$Y[,2],1.0/(1.0+exp(-lp)),log=TRUE))
     sim.chisq <- 2*(mixfit$logLik-binomial.loglik)
     fitno <<- fitno+1
     return(sim.chisq)
  }
  
  # fit glm to data
  binomial.mle <- glm.fit(object$X, object$Y,
                          offset = object$offset, family = binomial())
  
  
  thedata <- list(X=object$X,Y=object$Y,offset=object$offset,mle=binomial.mle)
  fitno <- 1
  binomial.boot <- boot(thedata, binomial.fun, R, sim="parametric",
                       ran.gen = binomial.rg, mle=binomial.mle, parallel=parallel,
                       ncpus = cores)
  return(binomial.boot)
}

outlierTest <-
  ## Short form for generic function
  function(object, R = 999, cores = max(detectCores(logical = FALSE)-1, 1))
    UseMethod("outlierTest")

outlierTest.robmixglm <- function(object, R = 999, cores = max(detectCores(logical = FALSE)-1, 1)) {
  if (!inherits(object, "robmixglm"))
    stop("Use only with 'robmixglm' objects.\n")
  
  if (cores > max(detectCores(logical = FALSE)-1, 1)) {
    cores <- max(detectCores(logical = FALSE)-1, 1)
    warning(sprintf("cores greater than available, setting to %i cores",max(detectCores(logical = FALSE)-1, 1)))
  }
  
  if (cores>1) {
    if(.Platform$OS.type=="unix") parallel <- "multicore"
    else parallel <- "snow"
  } else parallel <- "no"
  out <- switch (
    object$family,
    gaussian = gaussian.outlierTest.robmixglm(object, R, parallel, cores),
    binomial = binomial.outlierTest.robmixglm(object, R, parallel, cores),
    poisson = poisson.outlierTest.robmixglm(object, R, parallel, cores),
    gamma = gamma.outlierTest.robmixglm(object, R, parallel, cores),
    truncpoisson = truncpoisson.outlierTest.robmixglm(object, R, parallel, cores),
    nbinom = nbinom.outlierTest.robmixglm(object, R, parallel, cores)
  )
  class(out) <- "outlierTest"
  return(out)
}

summary.robmixglm <- function(object, ...) {
  if (!inherits(object, "robmixglm"))
    stop("Use only with 'robmixglm' objects.\n")
  out <- list()
  out$logLik <- logLik(object)
  out$AIC <- AIC(object)
  out$BIC <- BIC(object)
  coefficients <- matrix(object$fit@coef,ncol=1)
  coefficients[,1] <- ifelse(!(object$coef.names=="Outlier p."),coefficients[,1],1.0/(1.0+exp(-coefficients)))
  lastnames <- 2
  if (object$family %in% c("gaussian","gamma","nbinom")) lastnames <- 3
  coef.se <- sqrt(diag(object$fit@vcov))
  coef.se[(length(coefficients[,1])-lastnames+1):length(coefficients[,1])] <- NA
   coefficients <- cbind(coefficients,coef.se)
  
  coefficients <- cbind(coefficients,coefficients[,1]/coefficients[,2])
  coefficients <- cbind(coefficients,2*(1-pnorm(abs(coefficients[,3]))))
  dimnames(coefficients)[[1]] <- object$coef.names
  dimnames(coefficients)[[2]] <- c("Estimate","Std. Error","z value","Pr(>|z|)")
  out$coefficients <- coefficients
  class(out) <- "summary.robmixglm"
  out
}

logLik.robmixglm <- function(object, ...)
{
     val <- object$logLik
    attr(val, "df") <- object$np
    attr(val, "nobs") <- object$nobs
    class(val) <- "logLik"
    val
}

BIC.robmixglm <-
function (object, ...) 
{
    if (!is.element("robmixglm", class(object))) 
        stop("Argument 'object' must be an object of class \"robmixglm\".")
    BIC(logLik(object))
}

AIC.robmixglm <-
function(object,...,k=2) {
    if (!inherits(object, "robmixglm"))
        stop("Use only with 'robmixglm' objects.\n")
    AIC(logLik(object),...,k)
}

extractAIC.robmixglm <- function(fit, scale = 0, k = 2, ...)
{
  if (!inherits(fit, "robmixglm"))
    stop("Use only with 'robmixglm' objects.\n")
  n <- fit$nobs
  edf <- n  - fit$np # assumes dispersion is known
  aic <- AIC(fit)
  c(edf, aic + (k-2) * edf)
}

robmixglm <-
 function(formula,family=c("gaussian","binomial","poisson","gamma","truncpoisson","nbinom"),data,offset=NULL,quadpoints=21,
          notrials=50,EMTol=1.0e-5, cores = max(detectCores(logical = FALSE) %/% 2, 1), verbose=FALSE) {
 
   is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol
  
  if (cores > max(detectCores(logical = FALSE)-1, 1)) {
    cores <- max(detectCores(logical = FALSE)-1, 1)
    warning(sprintf("cores greater than available, setting to %i cores",max(detectCores(logical = FALSE)-1, 1)))
  }
  
  call <- match.call()

  if (missing(family)) family <- "gaussian"
  
  if (!(family %in% c("gaussian","binomial","poisson","gamma","truncpoisson","nbinom")))
    stop("Valid families are gaussian, binomial, poisson, gamma, truncpoisson, nbinom.\n")
  
  if (missing(data)) data <- environment(formula)
  
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("formula","data","offset"), names(mf), 0L)
  mf <- mf[c(1L,m)]
  mf$drop.unused.levels <- TRUE
  mf[[1L]] <- quote(stats::model.frame)
  mf <- eval(mf, parent.frame())
  mt <- attr(mf, "terms")

  Y <- model.response(mf, "any")

  X <- model.matrix(mt,mf)

  offset <- model.extract(mf,"offset")

  if(is.null(offset)) offset <- rep(0.0,dim(X)[1])
    
  if (family=="binomial")  {
    if (!inherits(Y,"matrix")) stop("Binomial data must be in success failure form.")
    if (dim(Y)[2]!=2) stop("Binomial data must be in success failure form.")
    if (any(Y[,1] <0) | any(Y[,2] <0)) stop("Binomial data must be positive.")
    if (any(!is.wholenumber(Y[,1]) | !is.wholenumber(Y[,2]))) stop("Binomial data must be integers.")
   } else if ((!inherits(Y,"numeric") & !inherits(Y,"integer"))) stop("Data must be a single column") 
  
  if (family=="poisson") {
    if (any(Y <0)) stop("Poisson data must be positive.")
    if (any(!is.wholenumber(Y))) stop("Poisson data must be integers.")
  }
  
  if (family=="gamma") {
    if (any(Y <0)) stop("Gamma data must be positive.")
  }
  
  if (family=="truncpoisson") {
    if (any(Y<=0)) stop("Truncated poisson data must be positive non-zero.")
    if (any(!is.wholenumber(Y))) stop("Truncated poisson data must be integers.")
  }

  if (family=="nbinom") {
    if (any(Y <0)) stop("Negative binomial data must be positive.")
    if (any(!is.wholenumber(Y))) stop("Negative binomial data must be integers.")
  }
  
  if (verbose & (cores>1)){
    warning("Setting cores=1 to allow Verbose output")
    cores <- 1
  }
  
  ret <- fit.robmixglm(X,Y,family,offset=offset,gh=norm.gauss.hermite(quadpoints),notrials,EMTol,cores,verbose)

  class(ret) <- "robmixglm"

  ret$call <- call
  ret$family <- family
  
  ret$X <- X
  ret$Y <- Y
  ret$offset <- offset
  
  ret$model <- mf
  ret$terms <- mt

  ret$xlevels <- .getXlevels(mt, mf)

  ret$quadpoints <- quadpoints
  ret$notrials <- notrials
  ret$EMTol <- EMTol
  ret$verbose <- verbose
  return(ret)
}

predict.robmixglm <-
  function(object, newdata = NULL, type = c("link", "response"), ...)
{
  if (!inherits(object, 'robmixglm'))
    stop("Primary argument much be a robmixglm object")

    call <- match.call()

    if (missing(type)) type <- "link"
    
     if(is.null(newdata)) {
       lp <- object$X %*% matrix(coef(object),ncol=1) + object$offset
       if (type=="link") return(lp)
       else return(switch(
         object$family,
         gaussian = lp,
         binomial = 1.0/(1.0+exp(-lp)),
         poisson = exp(lp),
         gamma = exp(lp),
         truncpoisson = exp(lp),
         nbinom = exp(lp)
       ))
     }
    Terms <- delete.response(terms(object))
    m <- model.frame(Terms, newdata, 
                     xlev = object$xlevels)
    X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
    offset <- rep(0, nrow(X))
    # if (!is.null(off.num <- attr(terms(object), "offset")))
    #   for(i in off.num)
    #     offset <- offset + eval(attr(terms(object), "variables")[[i+1]], newdata)
    if (!is.null(object$call$offset))
      offset <- offset + eval(object$call$offset, newdata)
    
     lp <- as.vector(X %*% matrix(coef(object),ncol=1))+offset
     if (type=="link") return(lp)
     else return(switch(
       object$family,
       gaussian = lp,
       binomial = 1.0/(1.0+exp(-lp)),
       poisson = exp(lp),
       gamma = exp(lp),
       truncpoisson = exp(lp),
       nbinom = exp(lp)
     ))
  }

print.outlierTest <- function(x, ...) {
  if (!inherits(x, "outlierTest"))
    stop("Use only with 'outlierTest' objects.\n")
  thestat <- x$t0[1]
  thestat <- ifelse(thestat<0.0,0.0,thestat)
  nullstat <- x$t[,1]
  nullstat <- ifelse(nullstat<0.0,0.0,nullstat)
  pos <- sum(nullstat>thestat)
  test <- (pos+1)/(length(x$t[,1])+1)
  out <- sprintf("p value %1.4f", test)
  cat(out)
  invisible(out)
}

summary.outlierTest <- function(object,...) {
  if (!inherits(object, "outlierTest"))
    stop("Use only with 'outlierTest' objects.\n")
  class(object) <- "summary.outlierTest"
  return(object)
}

print.summary.outlierTest <- function(x,...) {
  if (!inherits(x, "summary.outlierTest"))
    stop("Use only with 'summary.outlierTest' objects.\n")
  thestat <- x$t0[1]
  thestat <- ifelse(thestat<0.0,0.0,thestat)
  nullstat <- x$t[,1]
  nullstat <- ifelse(nullstat<0.0,0.0,nullstat)
  pos <- sum(nullstat>thestat)
  test <- (pos+1)/(length(x$t[,1])+1)
  out <- sprintf("p value %1.4f", test)
  cat(out)
  invisible(out)
}

gaussian.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TRUE, cores, verbose,   starting.values=NULL) {
  
  if (!is.null(starting.values)) {
    notrials <- 1
    cores <- 1
  }
  
  #browser()
  
  ll.robustgaussian <- function(p){
    
    xcoef <- p[1:(length(p)-3)]
    lpoutlier <- p[length(p)-2]
    sigma2 <- p[length(p)-1]
    sigma2out <- p[length(p)]
    
    poutlier <- 1.0/(1+exp(-lpoutlier))
    
    lp <- as.vector(x %*% xcoef)+offset
    
    ll1 <- dnorm(y, mean=lp, sd=sqrt(sigma2), log = TRUE)+log(1-poutlier)
    
    ll2 <- dnorm(y, mean=lp, sd=sqrt(sigma2out), log = TRUE)+log(poutlier)
    
    ll <- cbind(ll1,ll2)
    maxll <- apply(ll,1,max)
    negll <- -sum(maxll+log(apply(exp(ll-maxll),1,sum)))
    if (is.nan(negll)) negll <- NA
    if (!is.finite(negll)) negll <- NA
    return(negll)
  }
  
  thestatistic <- function(data, ...) {
    
    #browser()
    
    rlreg <- function(xcoef,lpoutlier,sigma2, sigma2out,x,y,offset,prop) {
      poutlier <- 1.0/(1+exp(-lpoutlier))
      
      lp <- as.vector(x %*% xcoef)+offset
      
      lp1 <- dnorm(y, mean=lp, sd=sqrt(sigma2), log = TRUE)
      lp2 <- dnorm(y, mean=lp, sd=sqrt(sigma2out), log=TRUE)
      
      if (!missing(prop)) {
        ll <- prop*cbind(lp1,lp2)
        negll <- -sum(apply(ll,1,sum))
      } else {
        l <- exp(cbind(lp1+log(1-poutlier),lp2+log(poutlier)))
        negll <- -sum(log(apply(l,1,sum)))
      }
      if (is.nan(negll)) negll <- NA
      if (!is.finite(negll)) negll <- NA
      return(negll)
    }
    
    optimrlreg <- function(p,lpoutlier,x,y,offset,prop) {
      return(rlreg(matrix(p[1:(length(p)-2)],ncol=1),lpoutlier,p[length(p)-1],p[length(p)],x,y,offset,prop))
    }
    #browser()
    
    y <- data[,1]
    x <- data[,3:dim(data)[2]]
    if (dim(data)[2] >=3) offset <- data[,2]
    else offset <- NULL
    
    tryCatch({
      if (is.null(starting.values)) {
 
      noutliers <- max(1,round(dim(data)[1]*0.2))
      outliers <- sample(c(rep(1,noutliers),rep(0,dim(data)[1]-noutliers)))

      robust.gaussian.prefit <- lm(y~-1+x,offset=offset,subset=(outliers!=1))
      
      prefit.coef <- coef(robust.gaussian.prefit)
      # assume 20% outliers as a starting point
      currlpoutlier <- log(0.2/(1-0.2))
      currxcoef <- matrix(prefit.coef[1:(length(prefit.coef))],ncol=1)
      currxcoef <- ifelse(is.na(currxcoef),0,currxcoef)
      currsigma2 <- summary(robust.gaussian.prefit)$sigma^2
      currsigma2out <- min(rgamma(1,2,1),5)*summary(robust.gaussian.prefit)$sigma^2
    } else {
      currxcoef <- matrix(starting.values[1:(length(starting.values)-3)],ncol=1)
      currlpoutlier <- starting.values[length(starting.values)-2]
      currsigma2 <- starting.values[length(starting.values)-1]
      currsigma2out <- starting.values[length(starting.values)]
    }
    currll <- -1.0e100
    nem <- 0
    
    repeat {
      nem <- nem+1
      # expectation step
      currpoutlier <- 1.0/(1+exp(-currlpoutlier))
      
      lp <- as.vector(x %*% currxcoef)+offset
      
      ll1 <- dnorm(y, mean=lp, sd=sqrt(currsigma2), log = TRUE)+log(1-currpoutlier) 
      ll2 <- dnorm(y, mean=lp, sd=sqrt(currsigma2out),log=TRUE)+log(currpoutlier)
      
      ll <- cbind(ll1,ll2)
      prop <- t(apply(ll,1,function(x) {
        x <- x-max(x)
        x <- ifelse(x==-Inf,-1e100,x)
        return(exp(x)/sum(exp(x)))
      }))
      # calculate outlier proportion
      poutlier <- sum(prop[,2])/dim(prop)[1]
      currlpoutlier <- log(poutlier/(1-poutlier))
      
      if (is.na(poutlier)) stop()
      
      startvals <- c(currxcoef,currsigma2,currsigma2out)
      names(startvals) <- c(dimnames(x)[[2]],"sigma2out","sigma2")
      
      results.nlm <- suppressWarnings(nlminb(startvals,optimrlreg,
                                             lower=c(rep(-Inf,length(currxcoef)),rep(0,2)),
                                             #control=list(trace=1,iter.max=5),
                                             control=list(trace=0,iter.max=5),
                                             lpoutlier=currlpoutlier,
                                             prop=prop,
                                             y=y,x=x,offset=offset))
      
      currxcoef <- matrix(as.numeric(results.nlm$par)[1:(length(results.nlm$par)-2)],ncol=1)
      currsigma2 <- as.numeric(results.nlm$par)[length(results.nlm$par)]
      currsigma2out <- as.numeric(results.nlm$par)[length(results.nlm$par)-1]
      
      if (currsigma2out < currsigma2) {
        temp <- currsigma2out
        currsigma2out <- currsigma2
        currsigma2 <- temp
      }
      
      lastll <- currll
      currll <- -rlreg(currxcoef,currlpoutlier,currsigma2,currsigma2out,x,y,offset)
      if (verbose) print(sprintf("Likelihood at end of EM step %.4f", currll))
      if (abs((lastll-currll)/currll)<EMTol) break()
      if (nem >100) break()
    }
    return(c(currll,currxcoef,currlpoutlier,currsigma2,currsigma2out))   
    },
    error=function(e) return(rep(NA,5))
    )
  }
  
  if (is.null(starting.values)) {
  bootdata <- cbind(y, offset, x)
  
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
  seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  
  
  if (cores>1) {
    if(.Platform$OS.type=="unix")  parallel <- "multicore"
    else parallel <- "snow" }
  else parallel <- "no"
   if (notrials == 1) {
    theboot <- matrix(thestatistic(bootdata), nrow=1)
  } else 
  theboot <- boot(bootdata, thestatistic, R=notrials, sim = "parametric",
                  ran.gen = function(d, p) d,
                  parallel = parallel,
                  ncpus = cores)$t

  res <- vector("list", length = notrials)
  for (i in 1:notrials) {
    res[[i]] <- list(logLik=theboot[i,1],start.val=theboot[i,2:dim(theboot)[2]])
  }
  
  maxll <- -Inf
  nfails <- 0
  
  #browser()
  for (i in 1:notrials) {
    if (verbose) cat(c(res[[i]]$logLik,res[[i]]$start.val),"\n")
    if (is.na(res[[i]]$logLik)) nfails <- nfails+1
    else {
      if (res[[i]]$logLik>maxll) {
        maxll <- res[[i]]$logLik
        start.val <- res[[i]]$start.val
      }
    }
    if (nfails > 0) warning(sprintf("Failed to obtain starting values for %i starting sets", nfails))
  }
  } else {
    start.val <- starting.values
  }
  
#browser()

  if(is.null(start.val)) stop("Cannot find valid starting values") 
  
  thenames <- c(dimnames(x)[[2]],"lpoutlier","sigma2out","sigma2")
  
  names(start.val) <- thenames
  
  parnames(ll.robustgaussian) <- names(start.val)
  
  lower.val <- c(rep(-Inf,length(start.val)-3),-Inf,0,0)
  
  names(lower.val) <- names(start.val)
  
  
  if (verbose) thecontrol <- list(eval.max=1000,iter.max=1000,eval.max=2000,trace=5)
  else thecontrol <- list(eval.max=1000,iter.max=1000,eval.max=2000)
  
  robustgaussian.fit <- mle2(ll.robustgaussian,start=start.val,vecpar=TRUE,
                             optimizer="user",optimfun=myoptim,
                             data=list(y=y,x=x,offset=offset),
                             skip.hessian=TRUE,trace=verbose,
                             lower=lower.val,
                             control=thecontrol)
  
  if (calcHessian) {
    thecoef <- coef(robustgaussian.fit)
    ncoef <- length(thecoef)
    if (thecoef[ncoef]<1e-4) thecoef[ncoef] <- 1e-4
    robustgaussian.fit@details$hessian <- optimHess(thecoef,ll.robustgaussian,control=list(ndeps=c(rep(1.0e-5,length(thecoef)))))
    robustgaussian.fit@vcov <- ginv(robustgaussian.fit@details$hessian)
  }
 
  xcoef <- matrix(coef(robustgaussian.fit)[1:(length(coef(robustgaussian.fit))-3)],ncol=1)
  lpoutlier <- coef(robustgaussian.fit)[length(coef(robustgaussian.fit))-2]
  poutlier <- 1.0/(1+exp(-lpoutlier))
  sigma2 <- coef(robustgaussian.fit)[length(coef(robustgaussian.fit))-1]
  sigma2out <- coef(robustgaussian.fit)[length(coef(robustgaussian.fit))]
  
  
  lp <- as.vector(x %*% xcoef)+offset
  
  ll1 <- dnorm(y, mean=lp, sd=sqrt(sigma2),log = TRUE)+log(1-poutlier) 
  ll2 <- dnorm(y, mean=lp, sd=sqrt(sigma2out),log = TRUE)+log(poutlier)
  
  ll <- cbind(ll1,ll2)
  prop <- t(apply(ll,1,function(x) {
    x <- x-max(x)
    x <- ifelse(x==-Inf,-1e100,x)
    return(exp(x)/sum(exp(x)))
  }))
  
  coef.names <- c(dimnames(x)[[2]],"Outlier p.","Sigma-sq", "Sigma-sq Out.")
  return(list(fit=robustgaussian.fit,prop=prop,logLik=-robustgaussian.fit@min,np=length(coef.names),nobs=dim(x)[1],coef.names=coef.names))  
} 

outlierProbs <-
  ## Short form for generic function 
  function(object) UseMethod("outlierProbs")

outlierProbs.robmixglm <- function(object) {
  if (!inherits(object, "robmixglm"))
    stop("Use only with 'robmixglm' objects.\n")
  outliers <- object$prop[,2]
  class(outliers) <- "outlierProbs"
  return(outliers)
}

`norm.gauss.hermite` <-
  function(n){
    rule <- gaussHermiteData(n)
    r <- cbind(rule$x,rule$w)  
    r[,1] <- r[,1]*sqrt(2)
    r[,2] <- r[,2]/sqrt(pi)
    #	r[,2] <- r[,2]/sum(r[,2])
    return(r)
  }

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

llrandpoiscpp <- function(y, lp, tau2, gh) {
    .Call(`_robmixglm_llrandpoiscpp`, y, lp, tau2, gh)
}

llrandtruncpoiscpp <- function(y, lp, tau2, gh) {
    .Call(`_robmixglm_llrandtruncpoiscpp`, y, lp, tau2, gh)
}

llrandgammacpp <- function(y, lp, tau2, phi, gh) {
    .Call(`_robmixglm_llrandgammacpp`, y, lp, tau2, phi, gh)
}

llrandbinomcpp <- function(y, lp, tau2, gh) {
    .Call(`_robmixglm_llrandbinomcpp`, y, lp, tau2, gh)
}

utils::globalVariables(c("robust.binomial.prefit","robust.gamma.prefit","robust.gaussian.prefit","robust.poisson.prefit","robust.truncpoisson.prefit"))
utils::globalVariables(c("robust.nbinom.prefit","gaussian.outlierTest.robmixglm","truncpoisson.mle2","gaussian.fun","gaussian.rg"))
utils::globalVariables(c("nbinom","nbinom.mle","offset","negbin.mle","nbinom.mle2"))

fit.robmixglm <- function(x,y,family,offset,gh=norm.gauss.hermite(21),notrials,EMTol,cores,verbose) {

 out <- switch(family,
          gaussian = gaussian.fit.robmixglm(x,y,offset,gh,notrials,EMTol,calcHessian=TRUE,cores,verbose,starting.values=NULL),
          binomial = binomial.fit.robmixglm(x,y,offset,gh,notrials,EMTol,calcHessian=TRUE,cores,verbose,starting.values=NULL),
          poisson = poisson.fit.robmixglm(x,y,offset,gh,notrials,EMTol,calcHessian=TRUE,cores,verbose,starting.values=NULL),
         gamma = gamma.fit.robmixglm(x,y,offset,gh,notrials,EMTol,calcHessian=TRUE,cores,verbose,starting.values=NULL),
         truncpoisson = truncpoisson.fit.robmixglm(x,y,offset,gh,notrials,EMTol,calcHessian=TRUE,cores,verbose,starting.values=NULL),
         nbinom = nbinom.fit.robmixglm(x,y,offset,gh,notrials,EMTol,calcHessian=TRUE,cores,verbose,starting.values=NULL)
 )
  out
}

coef.robmixglm <- function(object, ...) {
  if (!inherits(object, "robmixglm"))
    stop("Use only with 'robmixglm' objects.\n")
  res <- switch (
    object$family,
    gaussian = object$fit@coef[1:(length(object$fit@coef)-3)],
    binomial = object$fit@coef[1:(length(object$fit@coef)-2)],
    poisson = object$fit@coef[1:(length(object$fit@coef)-2)],
    gamma = object$fit@coef[1:(length(object$fit@coef)-3)],
    truncpoisson = object$fit@coef[1:(length(object$fit@coef)-2)],
    nbinom = object$fit@coef[1:(length(object$fit@coef)-3)]
    )
    names(res) <- object$coef.names[1:length(res)]
    res
}

residuals.robmixglm <-
  function(object,
           type = c("deviance", "pearson"),
           ...)
  {
    deviance.residuals <- function(){
      calc.binomial <- function(){
        n <- object$Y[,1]+object$Y[,2]
        sign(object$Y[,1]-n*fitted(object))*sqrt(
          2*(object$Y[,1]*log(object$Y[,1]/(n*fitted(object)))+
             object$Y[,2]*log(object$Y[,2]/(n*(1-fitted(object))))))
      }
      res <- switch(object$family,
            gaussian=(object$Y-fitted(object))/sqrt(coef(object$fit)[length(coef(object$fit))]),
            binomial=calc.binomial(),
            poisson=sign(object$Y-fitted(object))*sqrt(2*(object$Y*log(object$Y/fitted(object))-(object$Y-fitted(object)))),
            gamma=sign(object$Y-fitted(object))*sqrt(2*(-log(object$Y/fitted(object))+(object$Y-fitted(object))/fitted(object))),
            truncpoisson=sign(object$Y-fitted(object))*sqrt(2*(object$Y*log(object$Y/fitted(object))-(object$Y-fitted(object))))
      )
      res
    }
    pearson.residuals <- function(){
      res <- switch(object$family,
           gaussian=(object$Y-fitted(object))/sqrt(coef(object$fit)[length(coef(object$fit))]),
           binomial=(object$Y[,1]-(object$Y[,1]+object$Y[,2])*fitted(object))/
             sqrt((object$Y[,1]+object$Y[,2])*fitted(object)*(1-fitted(object))),
           poisson=(object$Y-fitted(object))/sqrt(fitted(object)),
           gamma=(object$Y-fitted(object))/fitted(object),
           truncpoisson=sign(object$Y-fitted(object))/(sqrt(fitted(object)*exp(fitted(object))*(-1+exp(fitted(object))-fitted(object)))/(exp(fitted(object))-1))
      )
      res
    }
    type <- match.arg(type)
     res <- switch(type,
           deviance=deviance.residuals(),
           pearson=pearson.residuals()
     )
    res
  }

\name{AIC}
\alias{AIC.robmixglm}
\title{AIC for robmixglm object}
\description{
Returns AIC for a robmixglm object.
}
\usage{
\method{AIC}{robmixglm}(object,  \ldots,  k  =  2)
}
\arguments{
  \item{object}{robmixglm object}
  \item{\dots}{ additional argument; currently none is used. }
  \item{k}{penalty per parameter}
}
\value{
AIC
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
AIC(forbes.robustmix)
}
}
\author{Ken Beath}
\keyword{methods}

\name{robmixglm}
\alias{robmixglm}
\title{Fits a Robust Generalized Linear Model and Variants}
\description{
Fits robust generalized linear models and variants described in Beath (2018).
}
\usage{
robmixglm(formula, family = c("gaussian", "binomial", "poisson", 
"gamma", "truncpoisson", "nbinom"), data, offset = NULL, 
quadpoints = 21, notrials = 50, EMTol = 1.0e-5,
cores  =  max(detectCores(logical = FALSE) \%/\% 2, 1), verbose = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{formula}{Model formula}
  \item{family}{Distribution of response}
  \item{data}{Data frame from which variables are obtained}
  \item{offset}{Offset to be incorporated in the linear predictor.}
  \item{quadpoints}{Number of quadrature points used in the Gauss-Hermite integration.}
  \item{notrials}{Number of random starting values to be used for EM}
  \item{EMTol}{Relative change in likelihood for completion of EM algorithm before switching to quasi-Newton}
  \item{cores}{Number of cores to be used for parallel evaluation of starting values}
  \item{verbose}{Print out diagnostic information? This includes the likelihood and parameter estimates for each EM run.}
}
\value{
  robmixglm object. This contains
\item{fit}{Final model fit from quasi-Newton}
\item{prop}{Posterior probability of observation in each class}
\item{logLik}{final log likelihood}
\item{np}{Number of parameters}
\item{nobs}{Number of observations}
\item{coef.names}{Coefficient names}
\item{call}{Call to function}
\item{family}{Family of model to be fitted}
  \item{model}{model}
  \item{terms}{terms}
  \item{xlevels}{Levels for factors.}
  \item{quadpoints}{Number of quadrature points used in the Gauss-Hermite integration.}
  \item{notrials}{Number of random starting values to be used for EM}
  \item{EMTol}{Relative change in likelihood for completion of EM algorithm before switching to quasi-Newton}
\item{verbose}{Was verbose output requested?}
}
\details{
Fits robust generalized models assuming that data is a mixture of standard observations and outlier abservations, which belong to an overdispersed model (Beath, 2018). For binomial, Poisson, truncated Poisson and gamma, the overdispersed component achieved through including a random effect as part of the linear predictor, as described by Aitkin (1996).  For gaussian and negative binomial data the outlier component is also a gaussian and negative binomial model, respectively but with a higher dispersion. For gaussian this corresponds to a higher value of \eqn{\sigma^2} but for negative binomial this is a lower value of \eqn{\theta}.

The method used is a generalised EM. Random starting values are determined by randomly allocating observations to either the standard or outlier class for the first iteration of the EM. The EM is then run to completion for all sets of starting values. The best set of starting values is then used to obtain the final results using a quasi-Newton method. Where the overdispersed data is obtained using a random effect, the likelihood is obtained by integrating out the random effect using Gauss-Hermite quadrature.
}
\references{Beath, K. J. A mixture-based approach to robust analysis of generalised linear models,  Journal of Applied Statistics, 45(12), 2256-2268 (2018) DOI: 10.1080/02664763.2017.1414164

Aitkin, M. (1996). A general maximum likelihood analysis of overdispersion in generalized linear models. Statistics and Computing, 6, 251262. DOI: 10.1007/BF00140869}
\author{Ken Beath}
\examples{
\donttest{
if (requireNamespace("MASS", quietly = TRUE)) {
library(MASS)
data(forbes)
# note that I have used cores=1 to allow testing
# in practice this should be removed to use the default
forbes.robustmix <- robmixglm(100*log10(pres)~bp, data = forbes, cores = 1)
}
}}
\keyword{multivariate}

\name{robmixglm-package}
\alias{robmixglm-package}
\docType{package}
\title{
Fits random effects meta-analysis models including robust models
}
\description{
Robust generalized linear models (GLM) using a mixture method,  as described in Beath (2018) <doi:10.1080/02664763.2017.1414164>.
}
\section{The robmixglm function}{
This is the main function that allows fitting the models. The robmixglm objects may be tested for outliers using outlierTest. The results of test.outliers may also be plotted.
}
\examples{
\donttest{
# for the following cores is set to 1 to satisfy the CRAN testing requirements
# removing will reduce the time taken depending on number of cores available
# animal brain vs body weight
library(MASS)
data(Animals)
Animals$logbrain <- log(Animals$brain)
Animals$logbody <- log(Animals$body)
lm1 <- lm(logbrain~logbody, data = Animals)
lm2 <- robmixglm(logbrain~logbody, data = Animals, cores = 1)
plot(Animals$logbody, Animals$logbrain)
abline(lm1, col = "red")
abline(lm2, col = "green")
plot(outlierProbs(lm2))
outlierTest(lm2, cores  =  1)

# Forbes data on relationship between atmospheric pressure and boiling point of water
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(100*log10(pres)~bp, data = MASS::forbes, cores = 1)
summary(forbes.robustmix)
plot(outlierProbs(forbes.robustmix))
outlierTest(forbes.robustmix, cores  =  1)

# diabetes
diabdata.robustmix <- robmixglm(glyhb~age+gender+bmi+waisthip+frame, 
    data = diabdata, cores = 1)
summary(diabdata.robustmix)
# this will take about 5-10 minutes
diabdata.step <- step(diabdata.robustmix, glyhb~age+gender+bmi+waisthip+frame)
summary(diabdata.step)
plot(outlierProbs(diabdata.step))
outlierTest(diabdata.step, cores  =  1)

# Hawkins' data
library(forward)
data(hawkins)
hawkins.robustmix <- robmixglm(y~x1+x2+x3+x4+x5+x6+x7+x8,
    cores = 1, data=hawkins)
summary(hawkins.robustmix)
plot(outlierProbs(hawkins.robustmix))
outlierTest(hawkins.robustmix, cores = 1)

# carrot damage
library(robustbase)
data(carrots)
carrots.robustmix <- robmixglm(cbind(success, total-success)~logdose+factor(block), 
     family = "binomial", data = carrots, cores = 1)
summary(carrots.robustmix)
plot(outlierProbs(carrots.robustmix))
outlierTest(carrots.robustmix, cores  =  1)

# train derailment
library(forward)
data(derailme)
derailme$cYear <- derailme$Year-mean(derailme$Year)
derailme$TrainKm100 <- derailme$TrainKm*100.0
derailme.robustmix <- robmixglm(y~cYear+factor(Type), offset = log(TrainKm100),
    family = "truncpoisson", quadpoints = 51,  data = derailme, cores = 1)
summary(derailme.robustmix)
plot(outlierProbs(derailme.robustmix))
outlierTest(derailme.robustmix, cores  =  1)
       
# hospital costs
hospcosts.robustmix <- robmixglm(costs~adm+age+dest+ins+loglos+sex, family = "gamma", 
    data = hospcosts, cores = 1)
summary(hospcosts.robustmix)
plot(outlierProbs(hospcosts.robustmix))
outlierTest(hospcosts.robustmix, cores  =  1)
}
}
\author{
Ken Beath <ken.beath@mq.edu.au>
}
\references{Beath,  K. J. A mixture-based approach to robust analysis of generalised linear models,  Journal of Applied Statistics,  45(12),  2256-2268 (2018) DOI: 10.1080/02664763.2017.1414164}

\name{extractAIC}
\title{Extract AIC from a Fitted Model}
%
\alias{extractAIC.robmixglm}
\usage{
\method{extractAIC}{robmixglm}(fit,  scale,  k  =  2,  \dots)
}
\arguments{
  \item{fit}{fitted \code{robmixglm} model.}
 \item{scale}{ignored.}
 \item{k}{numeric specifying the \sQuote{weight} of the
   \emph{equivalent degrees of freedom} (\eqn{\equiv}{ = :} \code{edf})
   part in the AIC formula.}
 \item{\dots}{further arguments (currently unused).}
}
\description{
  Computes the (generalized) AIC for a fitted \code{robmixglm} model. Used in \code{step},  otherwise use \code{AIC}.
}

\author{Ken Beath}
\seealso{
  \code{\link{extractAIC}}, \code{\link{step}}
}
\examples{\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = MASS::forbes, cores = 1)
extractAIC(forbes.robustmix)
}}
\keyword{models}

\name{print.outlierTest}
\alias{print.outlierTest}
\alias{summary.outlierTest}
\alias{print.summary.outlierTest}
\title{Print an outlierTest object}
\description{
Print an outlierTest object.
}
\usage{
\method{print}{outlierTest}(x,  \ldots)
}
\arguments{
  \item{x}{outlierTest object}
  \item{\ldots}{further arguments (not currently used)}
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
summary(forbes.robustmix)
print(outlierTest(forbes.robustmix,  cores  =  1))
}
}
\author{Ken Beath}
\keyword{methods}

\name{hospcosts}
\alias{hospcosts}
\docType{data}
\title{Hospital Costs data}
\description{
Data for the analysis in Beath (2018),  previously analysed in Marazzi and Yohai (2004),  Cantoni and Ronchetti (2006) and Heritier et al (2009). The data is for 100 patients hospitalised at the Centre Hospitalier Universitaire Vaudois in Lausanne,  Switzerland for "medical back problems" (APDRG 243).
}
\usage{hospcosts}
\format{
  A data frame with 100 observations on the following 9 variables.
  \describe{
    \item{\code{id}}{patient id}
    \item{\code{costs}}{cost of stay in Swiss francs}
    \item{\code{los}}{length of stay in days}
    \item{\code{adm}}{admission type,  0 = planned,  1 = emergency}
    \item{\code{ins}}{insurance type,  0 = regular,  1 = private}
    \item{\code{age}}{age in years}
    \item{\code{sex}}{sex,  0 = female,  1 = male}
    \item{\code{dest}}{discharge destination,  0 = another health institution,  1 = home}
    \item{\code{loglos}}{log of length of stay}
  }
}
\source{
  Heritier et al (2009)
  }
\references{
Cantoni,  E.,  & Ronchetti,  E. (2006). A robust approach for skewed and heavy-tailed outcomes in the analysis of health care expenditures. Journal of Health Economics,  25(2),  198213. http://doi.org/10.1016/j.jhealeco.2005.04.010

Heritier,  S.,  Cantoni,  E.,  Copt,  S. and Victoria-Feser,  M-P (2009). Robust Methods in Biostatistics. Wiley.

Marazzi,  A.,  & Yohai,  V. J. (2004). Adaptively truncated maximum likelihood regression with asymmetric errors. Journal of Statistical Planning and Inference,  122(12),  271291. http://doi.org/10.1016/j.jspi.2003.06.011
}
\examples{
\donttest{
hospcosts.robustmix <- robmixglm(costs~adm+age+dest+ins+loglos+sex, family = "gamma", 
    data = hospcosts, cores = 1)
summary(hospcosts.robustmix)
}}
\keyword{datasets}

\name{summary.robmixglm}
\alias{summary.robmixglm}
\title{summaryficients for robmixglm object}
\description{
Returns summary for a robmixglm object.
}
\usage{
\method{summary}{robmixglm}(object,  \ldots)
}
\arguments{
  \item{object}{robmixglm object}
  \item{\dots}{ additional argument; currently none is used. }
}
\value{
summary
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
summary(forbes.robustmix)
}}
\author{Ken Beath}
\keyword{methods}

\name{diabdata}
\alias{diabdata}
\docType{data}
\title{Diabetes data}
\description{
Data from Heritier et al (2009),  originally from Harrell (2001, p379). This data was from a study of the prevalence of cardiovascular risk factors such as obesity and diabetes for African Americans. (Willems et al,  19997) Data was available for 403 subjects screened for diabetes,  reduced to 372 after removal of cases with missing data.   
}
\usage{diabdata}
\format{
  A data frame with 372 observations on the following 8 variables.
  \describe{
    \item{\code{glyhb}}{Glycosated haemoglobin (values above 7.0 are usually taken as a positive diagnosis of diabetes)}
    \item{\code{age}}{age in years}
    \item{\code{gender}}{male or female}
    \item{\code{bmi}}{body mass index in kg/m^2}
    \item{\code{waisthip}}{ratio of waist to hip measurement}
    \item{\code{frame}}{body frame,  small,  medium or large}
    \item{\code{stab.glu}}{glucose}
    \item{\code{location}}{location,  Buckingham or Louisa}
  }
}
\source{
  Heritier et al (2009)
  }
\references{
Harrell,  F.E. (2001). Regression Modeling Strategies: With Applications to Linear Models,  Logistic Regression and Survival Analysis. Springer.

Heritier,  S.,  Cantoni,  E.,  Copt,  S. and Victoria-Feser,  M-P (2009). Robust Methods in Biostatistics. Wiley.

Willems,  J.P.,  Saunders,  J.T.,  Hunt,  D.E. and Schorling,  J.B. (1997) Prevalence of coronary heart disease risk factors among rural blacks: A community-based study. Southern Medical Journal,  90:814-820.
}
\examples{
\donttest{
diabdata.robustmix <- robmixglm(glyhb~age+gender+bmi+waisthip+frame+location, 
         data = diabdata, cores = 1)
summary(diabdata.robustmix)

diabdata.step <- step(diabdata.robustmix, glyhb~age+gender+bmi+waisthip+frame+location, cores = 1)
summary(diabdata.step)
}}
\keyword{datasets}

\name{outlierProbs}
\alias{outlierProbs}
\alias{outlierProbs.robmixglm}
\title{
Calculate outlier probabilities for each observation.}
\description{
For the normal mixture random effect calculates the probability that each observation is an outlier based on the posterior probability of it being an outlier.}
\usage{
outlierProbs(object)
}
\arguments{
  \item{object}{
A metaplus object with a mixture (robust) random effects distribution.
}
}
\value{
\item{outlier.prob}{Posterior probability that each observation is an outlier}
}
\details{
The outlier probabilities are obtained as the posterior probabilities of each observation being an outlier based on the fitted mixture model.
}
\author{
Ken Beath <ken.beath@mq.edu.au>
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
outlierProbs(forbes.robustmix)
}}
\keyword{method}

\name{BIC}
\alias{BIC.robmixglm}
\title{BIC for robmixglm object}
\description{
Returns BIC for a robmixglm object.
}
\usage{
\method{BIC}{robmixglm}(object,  \ldots)
}
\arguments{
  \item{object}{robmixglm object}
  \item{\dots}{ additional argument; currently none is used. }
}
\value{
BIC
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
BIC(forbes.robustmix)
}}
\author{Ken Beath}
\keyword{methods}

\name{plot.outlierProbs}
\alias{plot.outlierProbs}
\title{
Plot outlier probabilities.
}
\description{
Plots the outlier probability for each observation,  from an outlierProbs object.
}
\usage{
\method{plot}{outlierProbs}(x,  ...)
}
\arguments{
  \item{x}{
outlierProbs object to be plotted
}
 \item{\ldots}{
additional parameters to plot}
}
\value{
Plot
}
\author{
Ken Beath <ken.beath@mq.edu.au>
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
plot(outlierProbs(forbes.robustmix))
}}
\keyword{method}

\name{coef}
\alias{coef}
\alias{coef.robmixglm}
\title{Coefficients for a robmixglm object}
\description{
Returns coefficients for a robmixglm object. Only the coefficients for the linear part of the model are returned. Additional coefficients may be obtained using summary().
}
\usage{
\method{coef}{robmixglm}(object,  \ldots)
}
\arguments{
  \item{object}{robmixglm object}
  \item{\dots}{additional argument; currently none is used. }
}
\value{
coef
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
coef(forbes.robustmix)
}}
\author{Ken Beath}
\keyword{methods}

\name{residuals.robmixglm}
\alias{residuals.robmixglm}
\title{Extract Model Residuals}
\usage{
\method{residuals}{robmixglm}(object,  type  =  c("deviance",  "pearson"),  \dots)
}
\arguments{
  \item{object}{an object for which the extraction of model residuals is
    meaningful.}
  \item{type}{Type of residual where valid types are deviance and pearson.}
  \item{\dots}{other arguments.}
}
\value{
  Residuals extracted from the object \code{object}.
}
\description{
  Extracts model residuals from objects returned by modeling functions.
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
BIC(forbes.robustmix)
plot(fitted(forbes.robustmix), residuals(forbes.robustmix))
}}
\keyword{models}

\name{logLik}
\alias{logLik.robmixglm}
\title{log Likelikelihood for robmixglm object}
\description{
Returns log Likelihood for a robmixglm object.
}
\usage{
\method{logLik}{robmixglm}(object,  \ldots)
}
\arguments{
  \item{object}{robmixglm object}
  \item{\dots}{ additional argument; currently none is used. }
}
\value{
The loglikelihood.
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
logLik(forbes.robustmix)
}}
\author{Ken Beath}
\keyword{methods}

\name{fitted.robmixglm}
\alias{fitted.robmixglm}
\title{
Fitted values.}
\description{
Calculates the fitted values.}
\usage{
\method{fitted}{robmixglm}(object, \ldots)
}
\arguments{
  \item{object}{
A robmixglm object with a mixture (robust) random effects distribution.}
  \item{\ldots}{Other parameters. (not used)}
}
\value{
A vector of the fitted values.
}
\author{
Ken Beath <ken.beath@mq.edu.au>
}
\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(bp~pres, data = forbes, cores = 1)
BIC(forbes.robustmix)
plot(fitted(forbes.robustmix), residuals(forbes.robustmix))
}}
\keyword{method}

\name{outlierTest}
\alias{outlierTest}
\alias{outlierTest.robmixglm}
\title{
Test for the presence of outliers.}
\description{
Uses the parametric bootstrap to test for the presence of outliers.}
\usage{
outlierTest(object,  R = 999,  cores  =  max(detectCores(logical = FALSE)-1, 1))
}
\arguments{
  \item{object}{
A robmixglm object with a mixture (robust) random effects distribution.
}
  \item{R}{number of bootstrap replications}
  \item{cores}{Number of cores to be used in parallel. Default is one less than available.}
}
\value{
An outlierTest object which is the object of class \dQuote{boot} returned by the call to \command{boot}.
}
\details{
Performs a parametric bootstrap to compare models with and without outliers.
}
\author{
Ken Beath <ken.beath@mq.edu.au>
}
\examples{
\donttest{
# note that I have used cores=1 to allow testing
# in practice this should be removed to use the default
hospcosts.robustmix <- robmixglm(costs~adm+age+dest+ins+loglos+sex, family = "gamma", 
    data = hospcosts, cores = 1)
summary(hospcosts.robustmix)
# note that I have used cores=1 to allow testing
# in practice this should be removed to use the default
summary(outlierTest(hospcosts.robustmix,  cores  =  1))
}}
\keyword{method}

\name{predict.robmixglm}
\alias{predict.robmixglm}
\title{Predict Method for robmixglm}
\usage{
\method{predict}{robmixglm}(object,  newdata  =  NULL, 
            type  =  c("link",  "response"),  \ldots)
}
\arguments{
  \item{object}{a fitted object of class inheriting from \code{robmixglm}.}
  \item{newdata}{optionally,  a data frame in which to look for variables with
    which to predict.  If omitted,  the fitted linear predictors are used.}
  \item{type}{the type of prediction required.  The default \code{link} is on the
    scale of the linear predictors,  while the alternative \code{response}
    is on the scale of the response variable.}  
  \item{\ldots}{Other parameters. (not used)}
}
\description{
  Obtains predictions from a fitted robust mixture generalized linear model object.
}
\details{
  If \code{newdata} is omitted the predictions are based on the data
  used for the fit.  In that case how cases with missing values in the
  original fit is determined by the \code{na.action} argument of that
  fit.  If \code{na.action  =  na.omit} omitted cases will not appear in
  the residuals,  whereas if \code{na.action  =  na.exclude} they will
  appear (in predictions and standard errors),  with residual value
  \code{NA}.  See also \code{\link{napredict}}.
}
\value{
  A vector predicted linear predictors or response. For \code{binomial} the
  resonse is the predicted proportion.
  
}

\examples{
\donttest{
library(MASS)
data(forbes)
forbes.robustmix <- robmixglm(100*log10(pres)~bp, data = forbes, cores = 1)
plot(forbes$bp, forbes$pres)
preddata <- data.frame(bp = seq(from  =  min(forbes$bp), to  =  max(forbes$bp),  by  =  0.01))
# convert to original scale
preddata$predpres <-10^(predict(forbes.robustmix, newdata = preddata)/100)
lines(preddata$bp, preddata$predpres, col = "red")
}}
\keyword{models}
